@@ -1,4 +1,4 @@
-# Copyright © 2016-2020 Medical Image Analysis Laboratory, University Hospital Center and University of Lausanne (UNIL-CHUV), Switzerland
+# Copyright © 2016-2021 Medical Image Analysis Laboratory, University Hospital Center and University of Lausanne (UNIL-CHUV), Switzerland
 #
 #  This software is distributed under the open-source license Modified BSD.
 
@@ -10,16 +10,15 @@ It encompasses a High Resolution mask refinement and an N4 global bias field cor
 
 import os
 
-from glob import glob
-
 from traits.api import *
 
 from nipype.utils.filemanip import split_filename
-# from nipype.interfaces.base import isdefined, CommandLine, CommandLineInputSpec
 from nipype.interfaces.base import traits, \
     TraitedSpec, File, InputMultiPath, OutputMultiPath, BaseInterface, BaseInterfaceInputSpec
 
 from pymialsrtk.interfaces.utils import run
+import nibabel as nib
+import numpy as np
 
 
 #######################
@@ -53,7 +52,7 @@ class MialsrtkRefineHRMaskByIntersectionOutputSpec(TraitedSpec):
 
 
 class MialsrtkRefineHRMaskByIntersection(BaseInterface):
-    """Runs the MIAL SRTK mask refinement module.
+    """Runs the MIALSRTK mask refinement module.
 
     It uses the Simultaneous Truth And Performance Level Estimate (STAPLE) by Warfield et al. [1]_.
 
@@ -258,67 +257,100 @@ class FilenamesGeneration(BaseInterface):
     def _run_interface(self, runtime):
 
         for stack in self.inputs.stacks_order:
-            # print(self.inputs.sub_ses + '_run-' + str(stack) + '_T2w_nlm_uni_bcorr_histnorm.nii.gz',
-            #       '    --->     ',
-            #       self.inputs.sub_ses + '_run-' + str(stack) + '_id-' + str(self.inputs.sr_id) + '_desc-preprocSDI_T2w.nii.gz')
-            self.m_substitutions.append((self.inputs.sub_ses + '_run-' + str(stack) + '_T2w_nlm_uni_bcorr_histnorm.nii.gz',
-                                  self.inputs.sub_ses + '_run-' + str(stack) + '_id-' + str(
-                                      self.inputs.sr_id) + '_desc-preprocSDI_T2w.nii.gz'))
+            self.m_substitutions.append(('_T2w_nlm_uni_bcorr_histnorm.nii.gz',
+                                        '_id-' + str(
+                                        self.inputs.sr_id) + '_desc-preprocSDI_T2w.nii.gz'))
 
             if not self.inputs.use_manual_masks:
-                # print(self.inputs.sub_ses + '_run-' + str(stack) + '_T2w_brainMask.nii.gz',
-                #       '    --->     ',
-                #       self.inputs.sub_ses + '_run-' + str(stack) + '_id-' + str(self.inputs.sr_id) + '_desc-brain_mask.nii.gz')
-                self.m_substitutions.append((self.inputs.sub_ses + '_run-' + str(stack) + '_T2w_brainMask.nii.gz',
-                                      self.inputs.sub_ses + '_run-' + str(stack) + '_desc-brain_mask.nii.gz'))
+                self.m_substitutions.append(('_brainExtraction0/', ''))
+                self.m_substitutions.append(('_brainExtraction1/', ''))
+                self.m_substitutions.append(('_brainExtraction2/', ''))
+                self.m_substitutions.append(('_brainExtraction3/', ''))
+                self.m_substitutions.append(('_brainExtraction4/', ''))
+                self.m_substitutions.append(('_brainExtraction5/', ''))
+                self.m_substitutions.append(('_brainExtraction6/', ''))
+                self.m_substitutions.append(('_brainExtraction7/', ''))
+                self.m_substitutions.append(('_brainExtraction8/', ''))
+                self.m_substitutions.append(('_brainExtraction9/', ''))
+                self.m_substitutions.append(('_T2w_brainMask.nii.gz',
+                                             '_id-' + str(
+                                            self.inputs.sr_id) + '_desc-brain_mask.nii.gz'))
+            else:
+                self.m_substitutions.append(('_T2w_mask.nii.gz',
+                                             '_id-' + str(
+                                            self.inputs.sr_id) + '_desc-brain_mask.nii.gz'))
 
-            # print(self.inputs.sub_ses + '_run-' + str(stack) + '_T2w_nlm_uni_bcorr_histnorm.nii.gz',
-            #       '    --->     ',
-            #       self.inputs.sub_ses + '_run-' + str(stack) + '_id-' + str(self.inputs.sr_id) + '_desc-preprocSR_T2w.nii.gz')
-            self.m_substitutions.append((self.inputs.sub_ses + '_run-' + str(stack) + '_T2w_uni_bcorr_histnorm.nii.gz',
-                                  self.inputs.sub_ses + '_run-' + str(stack) + '_id-' + str(
-                                      self.inputs.sr_id) + '_desc-preprocSR_T2w.nii.gz'))
 
-            # print(self.inputs.sub_ses + '_run-' + str(stack) + '_T2w_nlm_uni_bcorr_histnorm_transform_' + str(
-            #     len(self.inputs.stacks_order)) + 'V.txt',
-            #       '    --->     ',
-            #       self.inputs.sub_ses + '_run-' + str(stack) + '_id-' + str(
-            #           self.inputs.sr_id) + '_T2w_from-origin_to-SDI_mode-image_xfm.txt')
-            self.m_substitutions.append((self.inputs.sub_ses + '_run-' + str(stack) + '_T2w_nlm_uni_bcorr_histnorm_transform_' + str(
-                len(self.inputs.stacks_order)) + 'V.txt',
-                                  self.inputs.sub_ses + '_run-' + str(stack) + '_id-' + str(
-                                      self.inputs.sr_id) + '_T2w_from-origin_to-SDI_mode-image_xfm.txt'))
+            self.m_substitutions.append(('_T2w_desc-brain_', '_desc-brain_'))
 
-            # print(self.inputs.sub_ses + '_run-' + str(stack) + '_T2w_uni_bcorr_histnorm_LRmask.nii.gz',
-            #       '    --->     ',
-            #       self.inputs.sub_ses + '_run-' + str(stack) + '_id-' + str(self.inputs.sr_id) + '_T2w_desc-brain_mask.nii.gz')
-            self.m_substitutions.append((self.inputs.sub_ses + '_run-' + str(stack) + '_T2w_uni_bcorr_histnorm_LRmask.nii.gz',
-                                  self.inputs.sub_ses + '_run-' + str(stack) + '_id-' + str(
-                                      self.inputs.sr_id) + '_T2w_desc-brain_mask.nii.gz'))
+            self.m_substitutions.append(('_srtkMaskImage010/', ''))
+            self.m_substitutions.append(('_srtkMaskImage011/', ''))
+            self.m_substitutions.append(('_srtkMaskImage012/', ''))
+            self.m_substitutions.append(('_srtkMaskImage013/', ''))
+            self.m_substitutions.append(('_srtkMaskImage014/', ''))
+            self.m_substitutions.append(('_srtkMaskImage015/', ''))
+            self.m_substitutions.append(('_srtkMaskImage016/', ''))
+            self.m_substitutions.append(('_srtkMaskImage017/', ''))
+            self.m_substitutions.append(('_srtkMaskImage018/', ''))
+            self.m_substitutions.append(('_srtkMaskImage019/', ''))
+            self.m_substitutions.append((#self.inputs.sub_ses +
+                                        # '_run-' + str(stack) +
+                                         '_T2w_uni_bcorr_histnorm.nii.gz',
+                                        # self.inputs.sub_ses +
+                                         # '_run-' + str(stack) +
+                                         '_id-' + str(
+                                        self.inputs.sr_id) + '_desc-preprocSR_T2w.nii.gz'))
 
-        # print('SDI_' + self.inputs.sub_ses + '_' + str(len(self.inputs.stacks_order)) + 'V_rad1.nii.gz',
-        #       '    --->     ',
-        #       self.inputs.sub_ses + '_rec-SDI' + '_id-' + str(self.inputs.sr_id) + '_T2w.nii.gz')
-        self.m_substitutions.append(('SDI_' + self.inputs.sub_ses + '_' + str(len(self.inputs.stacks_order)) + 'V_rad1.nii.gz',
-                              self.inputs.sub_ses + '_rec-SDI' + '_id-' + str(self.inputs.sr_id) + '_T2w.nii.gz'))
+            self.m_substitutions.append(( '_T2w_nlm_uni_bcorr_histnorm_transform_' + str(
+                                        len(self.inputs.stacks_order)) + 'V.txt',
+                                        '_id-' + str(
+                                        self.inputs.sr_id) + '_mod-T2w_from-origin_to-SDI_mode-image_xfm.txt'))
 
-        # print('SRTV_' + self.inputs.sub_ses + '_' + str(len(self.inputs.stacks_order)) + 'V_rad1_gbcorr.nii.gz',
-        #       '    --->     ',
-        #       self.inputs.sub_ses + '_rec-SR' + '_id-' + str(self.inputs.sr_id) + '_T2w.nii.gz')
-        self.m_substitutions.append(('SRTV_' + self.inputs.sub_ses + '_' + str(len(self.inputs.stacks_order)) + 'V_rad1_gbcorr.nii.gz',
-                              self.inputs.sub_ses + '_rec-SR' + '_id-' + str(self.inputs.sr_id) + '_T2w.nii.gz'))
+            self.m_substitutions.append(('_run-' + str(stack) +
+                                         '_T2w_uni_bcorr_histnorm_LRmask.nii.gz',
+                                        '_run-' + str(stack) + '_id-' + str(
+                                        self.inputs.sr_id) + '_desc-brain_mask.nii.gz'))
 
-        # print('SRTV_' + self.inputs.sub_ses + '_' + str(len(self.inputs.stacks_order)) + 'V_rad1.json',
-        #       '    --->     ',
-        #       self.inputs.sub_ses + '_rec-SR' + '_id-' + str(self.inputs.sr_id) + '_T2w.json')
-        self.m_substitutions.append(('SRTV_' + self.inputs.sub_ses + '_' + str(len(self.inputs.stacks_order)) + 'V_rad1.json',
-                              self.inputs.sub_ses + '_rec-SR' + '_id-' + str(self.inputs.sr_id) + '_T2w.json'))
+        self.m_substitutions.append(('SDI_' + self.inputs.sub_ses + '_' +
+                                     str(len(self.inputs.stacks_order)) + 'V_rad1.nii.gz',
+                                     self.inputs.sub_ses + '_rec-SDI' + '_id-' +
+                                     str(self.inputs.sr_id) + '_T2w.nii.gz'))
+
+        self.m_substitutions.append(('SRTV_' + self.inputs.sub_ses + '_' +
+                                     str(len(self.inputs.stacks_order)) + 'V_rad1_gbcorr.nii.gz',
+                                    self.inputs.sub_ses + '_rec-SR' + '_id-' +
+                                     str(self.inputs.sr_id) + '_T2w.nii.gz'))
+
+        self.m_substitutions.append(('SRTV_' + self.inputs.sub_ses + '_' +
+                                     str(len(self.inputs.stacks_order)) + 'V_rad1.json',
+                                    self.inputs.sub_ses + '_rec-SR' + '_id-' +
+                                     str(self.inputs.sr_id) + '_T2w.json'))
 
-        # print(self.inputs.sub_ses + '_T2w_uni_bcorr_histnorm_srMask.nii.gz',
-        #       '    --->     ',
-        #       self.inputs.sub_ses + '_rec-SR' + '_id-' + str(self.inputs.sr_id) + '_T2w_desc-brain_mask.nii.gz')
         self.m_substitutions.append((self.inputs.sub_ses + '_T2w_uni_bcorr_histnorm_srMask.nii.gz',
-                                     self.inputs.sub_ses + '_rec-SR' + '_id-' + str(self.inputs.sr_id) + '_T2w_desc-brain_mask.nii.gz'))
+                                     self.inputs.sub_ses + '_rec-SR' +
+                                     '_id-' + str(self.inputs.sr_id) +
+                                     '_mod-T2w_desc-brain_mask.nii.gz'))
+
+        self.m_substitutions.append(('SRTV_' + self.inputs.sub_ses +
+                                     '_' + str(len(self.inputs.stacks_order)) +
+                                     'V_rad1_srMask.nii.gz',
+                                     self.inputs.sub_ses + '_rec-SR' +
+                                     '_id-' + str(self.inputs.sr_id) +
+                                     '_mod-T2w_desc-brain_mask.nii.gz'))
+
+        self.m_substitutions.append(('SRTV_' + self.inputs.sub_ses +
+                                     '_' + str(len(self.inputs.stacks_order)) +
+                                     'V_rad1.png',
+                                     self.inputs.sub_ses + '_rec-SR' +
+                                     '_id-' + str(self.inputs.sr_id) + '_T2w.png'))
+
+        self.m_substitutions.append(('motion_index_QC.png',
+                                     self.inputs.sub_ses + '_rec-SR' +
+                                     '_id-' + str(self.inputs.sr_id) + '_desc-motion_stats.png'))
+
+        self.m_substitutions.append(('motion_index_QC.tsv',
+                                     self.inputs.sub_ses + '_rec-SR' +
+                                     '_id-' + str(self.inputs.sr_id) + '_desc-motion_stats.tsv'))
 
         return runtime
 
@@ -329,19 +361,58 @@ class FilenamesGeneration(BaseInterface):
         return outputs
 
 
+class BinarizeImageInputSpec(BaseInterfaceInputSpec):
+    """Class used to represent inputs of the BinarizeImage interface."""
 
-def binarize_image(input_image):
-    import nibabel as nib
-    import os
-    from nipype.utils.filemanip import split_filename
+    input_image = File(desc='Input image filename to be binarized', mandatory=True)
 
-    im = nib.load(input_image)
 
-    out = nib.Nifti1Image(dataobj=(im.get_fdata() > 0.01).astype(int), affine=im.affine)
-    out._header = im.header
+class BinarizeImageOutputSpec(TraitedSpec):
+    """Class used to represent outputs of the BinarizeImage interface."""
 
-    _,name,ext = split_filename(input_image)
-    output_mask = name + '_srMask' + ext
-    nib.save(filename=output_mask, img=out)
+    output_srmask = File(desc='Image mask (binarized input)')
 
-    return os.path.abspath(output_mask)
\ No newline at end of file
+
+class BinarizeImage(BaseInterface):
+    """Runs the MIAL SRTK mask image module.
+    Example
+    =======
+    >>> from pymialsrtk.interfaces.postprocess import BinarizeImage
+    >>> maskImg = MialsrtkMaskImage()
+    >>> maskImg.inputs.input_image = 'input_image.nii.gz'
+    """
+
+    input_spec = BinarizeImageInputSpec
+    output_spec = BinarizeImageOutputSpec
+
+    def _gen_filename(self, name):
+        if name == 'output_srmask':
+            _, name, ext = split_filename(self.inputs.input_image)
+            output = name + '_srMask' + ext
+            return os.path.abspath(output)
+        return None
+
+    def _binarize_image(self, in_image):
+
+        image_nii = nib.load(in_image)
+        image = np.asanyarray(image_nii.dataobj)
+
+        out = nib.Nifti1Image(dataobj=1 * (image > 0), affine=image_nii.affine)
+        out._header = image_nii.header
+
+        nib.save(filename=self._gen_filename('output_srmask'), img=out)
+
+        return
+
+    def _run_interface(self, runtime):
+        try:
+            self._binarize_image(self.inputs.input_image)
+        except Exception as e:
+            print('Failed')
+            print(e)
+        return runtime
+
+    def _list_outputs(self):
+        outputs = self._outputs().get()
+        outputs['output_srmask'] = self._gen_filename('output_srmask')
+        return outputs
