@@ -110,3 +110,33 @@ def reorder_by_run_ids(p_files, p_order):
     #     id_and_files_ordered = id_and_files_ordered + remainings
 
     return [i[1] for i in id_and_files_ordered]
+
+
+def get_emission_car_miles_equivalent(emissions):
+    """Return the equivalent of CO2 emissions [Kg] in terms of kms traveled by an average car.
+
+    References
+    ----------
+    https://github.com/mlco2/codecarbon/blob/c6aebb9681186a71573748e381b6a3c9731de2d3/codecarbon/viz/data.py#L53
+
+    """
+    return "{:.0f}".format((emissions / 0.409) * 1.60934)
+
+
+def get_emission_tv_time_equivalent(emissions):
+    """Return the equivalent of CO2 emissions [Kg] in terms of kms traveled by an average car.
+
+    References
+    ----------
+    https://github.com/mlco2/codecarbon/blob/c6aebb9681186a71573748e381b6a3c9731de2d3/codecarbon/viz/data.py#L66
+
+    """
+    tv_time_in_minutes = emissions * (1 / 0.097) * 60
+    tv_time = "{:.0f} minutes".format(tv_time_in_minutes)
+    if tv_time_in_minutes >= 60:
+        time_in_hours = tv_time_in_minutes / 60
+        tv_time = "{:.0f} hours".format(time_in_hours)
+        if time_in_hours >= 24:
+            time_in_days = time_in_hours / 24
+            tv_time = "{:.0f} days".format(time_in_days)
+    return tv_time
